using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Reflection;
using System.Text;

namespace FlutterCodeGenerator
{
    internal class MethodMap
    {
        public List<ModelType> ParameterModeTypes;
        public ModelType ReturnParameterModelType { get; set; }
        private readonly string _methodName;

        public MethodMap(MethodInfo method)
        {
            ParameterModeTypes = new List<ModelType>();
            _methodName = method.Name;
            var returnParameterType = method.ReturnType.GetGenericArguments()[0];

            ReturnParameterModelType = ModelTypeGenerator.Create(returnParameterType, "");

            var parameters = method.GetParameters();
            foreach (var parameter in parameters)
            {
                var parameterType = parameter.ParameterType;
                var parameterModeType = ModelTypeGenerator.Create(parameterType, parameter.Name);
                ParameterModeTypes.Add(parameterModeType);
            }
        }

        public string GetMethodDartString()
        {
            var dartString = new StringBuilder();
            var parameterNames = string.Join(",", ParameterModeTypes.Select(r => r.GetFlutterTypeName() + " " + r.Name_Lower));
            var argumentNames = string.Join(",", ParameterModeTypes.Select(r => r.Name_Lower));
            if (ParameterModeTypes.Count > 1)
            {
                argumentNames = "[" + argumentNames + "]";
            }
            dartString.AppendLine($"\tFuture<{ReturnParameterModelType.GetFlutterTypeName()}> {LetterConverterHelper.FirstCharToLower(_methodName)}({parameterNames}) async {{");
            dartString.AppendLine($"\t\tvar rpcRst = await call(\"{_methodName}\", {argumentNames});");
            if (ReturnParameterModelType is SimpleModelType && !(ReturnParameterModelType is ListModelType) && !(ReturnParameterModelType is DictionaryModelType) && !(ReturnParameterModelType is ArrayModelType))
            {
                dartString.AppendLine("\t\treturn rpcRst;");
            }
            else if (ReturnParameterModelType is UserDefinedComplexModeType || ReturnParameterModelType is UserDefinedGenericComplexModelType)
            {
                dartString.AppendLine($"\t\tvar result = {ReturnParameterModelType.GetFlutterTypeName()}.fromJson(rpcRst as Map<String, dynamic>);");
                dartString.AppendLine("\t\treturn result;");
            }
            else if (ReturnParameterModelType is ListModelType genericModelType)
            {
                if (genericModelType.GenericArgumentModelType is SimpleModelType simpleModelType)
                {
                    dartString.AppendLine($"\t\tvar result = (rpcRst as List).cast<{simpleModelType.GetFlutterTypeName()}>().toList();");
                    dartString.AppendLine("\t\treturn result;");
                }
                else if (genericModelType.GenericArgumentModelType is UserDefinedComplexModeType userDefinedComplexModeType)
                {
                    dartString.AppendLine($"\t\tvar result = (rpcRst as List).map((e)=>{userDefinedComplexModeType.GetFlutterTypeName()}.fromJson(e as Map<String, dynamic>)).toList();");
                    dartString.AppendLine("\t\treturn result;");
                }
            }
            dartString.AppendLine("\t}");
            return dartString.ToString();
        }
    }
}
