using System;
using System.Linq;

namespace FlutterCodeGenerator
{
    public class ModelTypeGenerator
    {
        public static ModelType Create(Type type, string argName)
        {
            if (type == typeof(int) || type == typeof(long) || type == typeof(short))
            {
                return new IntModeType(argName);
            }
            else if (type == typeof(float) || type == typeof(double))
            {
                return new DoubleModeType(argName);
            }
            else if (type == typeof(bool))
            {
                return new BoolModeType(argName);
            }
            else if (type == typeof(string) || type == typeof(char))
            {
                return new StringModeType(argName);
            }
            else if (type == typeof(DateTime))
            {
                return new DateTimeModeType(argName);
            }
            else if (type.IsEnum)
            {
                var modelType = new EnumModeType(type, argName);
                if (!ServiceMap.TemporaryList.Any(x => x.ParameterType == type))
                {
                    ServiceMap.TemporaryList.Add(modelType);
                }
                return modelType;
            }
            else if (type.IsArray)
            {
                var modelType = new ArrayModelType(type, argName);
                return modelType;
            }
            else if (type.IsGenericType)
            {
                if (type.Name == "List`1")
                {
                    var modelType = new ListModelType(type, argName);
                    return modelType;
                }
                else if (type.Name == "Dictionary`2")
                {
                    var modelType = new DictionaryModelType(type, argName);
                    return modelType;
                }
                else
                {
                    var modelType = new UserDefinedGenericComplexModelType(type, argName);
                    if (!ServiceMap.TemporaryList.Any(x => x.ParameterType == type))
                    {
                        ServiceMap.TemporaryList.Add(modelType);
                    }
                    return modelType;
                }
            }
            else
            {
                var modelType = new UserDefinedComplexModeType(type, argName);
                if (!ServiceMap.TemporaryList.Any(x => x.ParameterType == type))
                {
                    ServiceMap.TemporaryList.Add(modelType);
                }
                return modelType;
            }
        }
    }
}
